--[[
	Collision class so I can keep main.lua clean
]]

local collision={}
collision.__index=collision
local Vector2=require("Modules/Vector2")

-- function collision:CheckCollision(obj1, obj2)
  -- return obj1.x < obj2.x+obj2.w and
         -- obj2.x < obj1.x+obj1.w and
         -- obj1.y < obj2.y+obj2.h and
         -- obj2.y < obj1.y+obj1.h
-- end

function collision:collidesMTV(obj1, obj2)
	--[[
		Adapated from(the person who did this, EgoMoose, has a lot of nice tutorials for ROBLOX Studio stuff that can be easily translated into love2d):
		
		http://wiki.roblox.com/index.php?title=2D_Collision_Detection
		
		Hopefully will stop the ball from being stuck inside the paddles in some cases but may allow for some weird collisions if the ball hits top/bottom of paddle.
	]]
	local g1p, g1s = {x=obj1.x,y=obj1.y}, {x=obj1.w,y=obj1.h};
	local g2p, g2s = {x=obj2.x,y=obj2.y}, {x=obj2.w,y=obj2.h};
	-- check for collision first
	local doesCollide, mtv = ((g1p.x < g2p.x + g2s.x and g1p.x + g1s.x > g2p.x) and (g1p.y < g2p.y + g2s.y and g1p.y + g1s.y > g2p.y));
	-- since we know they collide, find mtv
	if doesCollide then
		-- find distances between shape1's edge and shape2's opposite edge
		local edgeDifferences = {
			Vector2.new(g1p.x - (g2p.x + g2s.x), 0); -- left
			Vector2.new((g1p.x + g1s.x) - g2p.x, 0); -- right
			Vector2.new(0, g1p.y - (g2p.y + g2s.y)); -- top
			Vector2.new(0, (g1p.y + g1s.y) - g2p.y); -- bottom
		};
		-- get the smallest difference
		table.sort(edgeDifferences, function(a, b) return a.magnitude() < b.magnitude(); end);
		mtv = edgeDifferences[1];
	end;
	-- provide collision boolean and mtv
	return doesCollide, mtv or {};
end;

return collision